/***************************************************************************************
 * @file     reb_main.c
 * @version  V1.0
 * @date     31. March 2016
 *
 * @note
 * VORAGO Technologies 
 *
 * @note
 * Copyright (c) 2013-2016 VORAGO Technologies. 
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY 
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT. 
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES 
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL 
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
#include <stdio.h>
#include "va108xx.h"
#include "reb_log.h"
#include "reb_board.h"
#include "reb_adt75.h"
#include "driver_common.h"
#include "reb_max11619.h"
#include "reb_timer.h"
#include "gpio_va108xx.h"

//   *******  Global variables set here  *******
volatile uint8_t bounce_cnt = 0; 
         uint32_t G_TDO[32], EF_ADD_value[32]  ;   // storage for TDO data for each message    
				uint32_t time_to_prog ;  


//*********************************************************************************************
//    Project intended to drive JTAG port on another VA108xx MCU 
//     Port pin use table:
//     PB15 - TCK   (output)
//		 PB16 - TMS    (output)
//     PB18 - TRST   (output)
//     PB17 - TDI    (output)
//     PB13 - TDO    (input)


//*********************************************************************************************

#define sTCK   15
#define sTMS   16
#define sTRST  12
#define sTDI   17
#define sTDO	 13


//*************************************************************************
//  Routine to configure TIM3 for measuring subroutine duration
//    -  Will set timer to $FFFF:FFFF before starting measurement 
//    -  
//    -    
//    
//**************************************************************************

uint32_t CONFIG_TIM3(void)
{
			VOR_SYSCONFIG->TIM_CLK_ENABLE |= 0xFFFF  ;  //  enable clocks to lower 16 TIM blocks 
	
			
  
	    VOR_TIM3->CTRL = (TIM1_CTRL_IRQ_ENB_Msk | ( 2 << TIM1_CTRL_STATUS_SEL_Pos))  ;  //  set status select to 2 = toggle at countdown
			VOR_TIM3->RST_VALUE = 0xFFFFFFFF ; 
	    VOR_TIM3->CNT_VALUE = 0xFFFFFFFF ; 
	
		 
												// Setup cascade control so timer triggers when PA11(button) is pressed
//			VOR_TIM3->CASCADE0 = (TIM_CAS_SRC_PORTA_11) ;   // set cascade 0 source to PORTA11
//			VOR_TIM3->CASCADE1 = 0x00 ; // not used 
//			VOR_TIM3->CASCADE2 = 0x00 ; // not used 
//      VOR_TIM3->PWMA_VALUE = PWM_period * .6   ;  
//	    VOR_TIM3->PWMB_VALUE = PWM_period * .4  ; 	 
			VOR_TIM3->ENABLE = 0x1   ;    // enable TIM3
			
	return(1)  ; 
}

//   ****** subroutine to cycle TCK 1k times
//   clock will be high upon entry and exit. 

void TCK_1k(void)  
{
volatile	    uint32_t  cnt,cnt2   ; 
			
		for(cnt=0; cnt < 1000; cnt ++)
		{
			VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
	           for(cnt2=0; cnt2 < 10; cnt2 ++)  { __NOP() ;}   
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT shou
						 for(cnt2=0; cnt2 < 10; cnt2 ++)  { __NOP() ;} 
		}
	    
	return ; 
}

//   **  short delay  

void tclk_delay(void)
{
uint32_t  i  ;  

for(i=1;i<10;i++ )     
	{
			__NOP() ;  
	}//  delay here 

} 
//*********************************************************************************************
//    Subroutine to setup port pins and RESET the JTAG controller  
//        setup PA0-3 as output and PA4 as an input
//         drive reset pin of JTAG low for two TCLK cycles 
//         Move state machine to pause parking position (1 more clock with TMS = 0 
//         Data changes on falling edge of TCK and is sampled on rising edge. 


int32_t Reset_JTAG()  
{
//	uint32_t i   ;  
		VOR_GPIO->BANK[1].DIR |= (1 << sTCK)|(1 << sTMS)|(1 << sTRST)|(1 << sTDI) ; 
		VOR_GPIO->BANK[1].DIR &= ~(1<<sTDO)  ;  
	
		VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK)|(1 << sTMS)|(1 << sTRST)|(1 << sTDI);  // set all outputs to high state
	
		VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTRST);  // set trst low 
		tclk_delay()  ; 
		VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);   // set tclk low 
	  tclk_delay()  ;
	  VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);    // set tclk high -  DUT should process RST active here
		tclk_delay() ;  
	   VOR_GPIO->BANK[1].DATAOUT |= (1<<sTRST)  ;  	//  set TRST high 
	   VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);   // set tclk low 
		VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTMS);    // set TMS low 
	  tclk_delay()  ;
	  VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);     // set tclk high
	  tclk_delay()  ;
	  tclk_delay()  ;  //   add second delay just for debug purposes to show break
	return 0  ;  
}




uint32_t rev_bit_order (uint32_t num_bits, uint32_t in_data) 
{
volatile	  uint32_t out_data, temp1, mask   ; 
	   temp1 = 0  ;
	   mask = 1 ; 
	  out_data = 0   ;  
	  while( num_bits -- > 0 ) 
	  {
	  temp1 = (in_data & mask) ; 
		if(temp1 != 0)
			{
				 out_data |= 0x1  ;  //  set lsb to 1 
			} 
     out_data <<=  1 ;  //  shift left 
		 mask <<= 1   ;    //   shift mask 1 position left
	  }
//	  out_data >>= 1   ;   //   correct for one too many shifts left.  
	
 
   return out_data >> 1   ;  	
	 
}


//*********************************************************************************************
//    Subroutine to shift out data to data register of JTAG
//    input characters:  bitcnt - number of bits to shift  (1 to 65) 
//                       TDI_value - value to be shifted  (note:  LSBit is shifted out first)  
//    output character:  TDO_data -  This is the information clocked in on the TDO line during the shift.  
//

//    
uint32_t SDR_sub(uint32_t bitcnt, uint32_t TDI_value) 
{
volatile	 int32_t  TDO_data, TDO_bitpos, temp1  ; 
volatile		 int8_t  bit_out  ;  
    TDO_data = 0  ;  	
	  TDO_bitpos = 1   ;  
    temp1 = TDI_value  ;  
	
//  ** Move JTAG state machine position  (TMS =  1 0 0 to get from park to point where data is shifted) 

		  VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			VOR_GPIO->BANK[1].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			tclk_delay()  ;	
			VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTMS); 		   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			tclk_delay()  ;	
			VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
      tclk_delay()  ;
// ** Start shifting data out here  
	while (bitcnt-- > 0) 
	{
		  bit_out = TDI_value & 0x0001    ;
			TDI_value = TDI_value >> 1  ;    //   shift TDI_value one position to get ready for next bit
			if(bit_out == 1) 
			{
			       VOR_GPIO->BANK[1].DATAOUT |= (1 << sTDI);
			}
			else
			{
				    VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTDI); 
				
			}
       if(bitcnt == 0)  {		  VOR_GPIO->BANK[1].DATAOUT |= (1 << sTMS); } // set TMS high on last bit 
		  VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);  // set tclk low 
 
	    tclk_delay()  ; 
	    VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			temp1 = (VOR_GPIO->BANK[1].DATAIN & (1<<sTDO))  ;  //  sample TDO on rising edge of TCLK 
			if(temp1 != 0)
			{
				TDO_data |= TDO_bitpos ;  //  set bit in TDO if pin sample = 1, otherwise leave as zero 
			}
		
			TDO_bitpos = (  TDO_bitpos << 1 ) ;   //  move bit position to left 
		  tclk_delay() ;			
	}
	
	//  ** Move JTAG state machine position  (TMS =  1 0 to get from point where data is shifted to park) 

	     VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[1].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			 tclk_delay()  ;
	     VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
       tclk_delay()  ;
		return TDO_data ;  
}  


//  ****  trying to figure out why return statements are being optimized out  

uint32_t SIR_sub(uint32_t bitcnt, uint32_t TDI_value) 
{
volatile	 int32_t  TDO_data, TDO_bitpos, temp1  ; 
volatile		 int8_t  bit_out,out3  ;  
    TDO_data = 0  ;  	
		out3 = 0  ;  
	  TDO_bitpos = 1   ;  
    temp1 = TDI_value  ;  
	
//  ** Move JTAG state machine position  (TMS =  1 1 0 0 to get from park to point where data is shifted) 

		  VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			VOR_GPIO->BANK[1].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			tclk_delay()  ;	
			VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[1].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			tclk_delay()  ;	
			VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
      tclk_delay()  ;
			VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
      tclk_delay()  ;
// ** Start shifting data out here  
	while (bitcnt-- > 0) 
	{
		  bit_out = TDI_value & 0x0001    ;
			TDI_value = TDI_value >> 1  ;    //   shift TDI_value one position to get ready for next bit
			if(bit_out == 1) 
			{
			       VOR_GPIO->BANK[1].DATAOUT |= (1 << sTDI);
			}
			else
			{
				    VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTDI); 
				
			}
      if(bitcnt == 0)  {		  VOR_GPIO->BANK[1].DATAOUT |= (1 << sTMS); } // set TMS high on last bit 
		  VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);  // set tclk low 
 
	    tclk_delay()  ; 
	    VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			temp1 = (VOR_GPIO->BANK[1].DATAIN & (1<<sTDO))  ;  //  sample TDO on rising edge of TCLK 
			if(temp1 != 0)
			{
				TDO_data |= TDO_bitpos ;  //  set bit in TDO if pin sample = 1, otherwise leave as zero 
			}
		
			TDO_bitpos = (  TDO_bitpos << 1 ) ;   //  move bit position to left 
		  tclk_delay() ;			
	}
	
	//  ** Move JTAG state machine position  (TMS =   1 0 to get from point where data is shifted to park) 

	    VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			VOR_GPIO->BANK[1].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			tclk_delay()  ;
	    VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			VOR_GPIO->BANK[1].DATAOUT &= ~(1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[1].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
      tclk_delay()  ;
		return TDO_data ;  
}  

int32_t comm_timeout(uint32_t source)  
{
	volatile uint32_t t  ; 
	t = source   ;  //  source codes  3 = eFuse status never changed, 5 = prog value != read value
                   //  9 = wrong IDcode read   
	while(1) {   }    ;    //  An error has occurred!!!!   hang here forever    
  return(t)   ;  	
}

#define EF_ADDR 0x14
#define EF_WDATA 0x15
#define EF_RDATA 0x16
#define EF_TIMING 0x17
#define EF_CMD 0x18
#define EF_STATUS 0x19

//  *******************************************************************
//  Read efuse 
//    calling parameters  (count = number of location to read (0-31), start_add = start address (0-31)
//    Read values will be stored in a global array EF_ADD_value[] 
//  Per programmers guide 
//      step a - Load EF_WDATA with 0 
//			step b - Load EF_ADDR with desired read address
//			step c - Load EF_CMD with 0x1 
//      step d - Poll EF_STATUS for Bit0 to be 1 
//      step e - Load EF_CMD with 0x5  
//			step f - Read EF_RDATA 
//			step g - Load EF_CMD with 0x0 to disable the interface.  


uint32_t Read_EF_ADR(uint32_t add_count, uint32_t start_add)  
{
		uint8_t  HBO_ready, cnt=0, loop_count  ; 
volatile	   uint32_t read_add   ;  
	  loop_count = 0 ; 
	 while(loop_count++ < add_count) 
	 {
	  G_TDO[0] = SIR_sub(9,EF_WDATA)  ;   // step a
	  G_TDO[1] = SDR_sub(32,0x0) ;
	
		read_add = (start_add+loop_count-1)  ;  // step b
		G_TDO[2] = SIR_sub(9,EF_ADDR)  ; 
	  G_TDO[3] = SDR_sub(6,read_add) ;
		 
		G_TDO[4] = SIR_sub(9,EF_CMD)  ; // step c
	  G_TDO[5] = SDR_sub(4,0x1) ;
	
		G_TDO[20] = SIR_sub(9,EF_STATUS)  ; // step d
	  G_TDO[21] = SDR_sub(3,0x1) ;
		
		HBO_ready = 0 ; 
	  while(HBO_ready == 0) 
		{
			G_TDO[5] = SIR_sub(9,EF_STATUS)  ; 
			G_TDO[6] = SDR_sub(3,0x1) ;
			if(G_TDO[6] != 0)  HBO_ready = 1 ;
			 cnt ++ ;  
			 if (cnt > 100)  {  comm_timeout(1)  ;  }
		}
	
		G_TDO[7] = SIR_sub(9,EF_CMD)  ; // step e
	  G_TDO[8] = SDR_sub(4,0x5) ;
		G_TDO[8] = SDR_sub(4,0x3) ;  //   clear the start operation bit
	
		G_TDO[9] = SIR_sub(9,EF_RDATA)  ; // step f
	  G_TDO[10] = SDR_sub(32,0x0) ;
		EF_ADD_value[loop_count-1] = G_TDO[10]  ;  

		G_TDO[11] = SIR_sub(9,EF_CMD)  ; // step g
	  G_TDO[12] = SDR_sub(4,0x0) ;	
  }		 
    return cnt  ;  
	
}

//  *******************************************************************
//  Program efuse 
//    calling parameters  (address (0-31), value(32bits, any combination)  
//  Per programmers guide 
//       
//			Step a - Load EF_ADDR with desired write address "address"
//      Step b - Load EF_WDATA with value to be programmed "value"
//      Step c  - Load EF_TIMING with proper timing 
//			Step d - Load EF_CMD with 0x3 
//      Step e- Poll EF_STATUS for Bit0 to be 1 
//      Step f- bring write enable pin low
//      Step g - Load EF_CMD with 0x7 
//      Step h- Poll EF_STATUS for Bit1 to be 0
//      Step i- bring efuse write enable pin high
//			Step j- Load EF_CMD with 0x0 to disable the interface. 
//      Step k - read the location to verify program worked 

uint32_t Prog_EF_ADR(uint32_t address, uint32_t value)  
{
		uint8_t  HBO_ready, cnt=0,ctrl_busy  ; 
volatile	   uint32_t read_add, iter_til_prog_complete = 0   ;  
	
 	  Reset_JTAG()  	  ;  // reset JTAG statemachine				
		G_TDO[2] = SIR_sub(9,EF_ADDR)  ;  //  step a
	  G_TDO[3] = SDR_sub(6, address) ;
	
		G_TDO[0] = SIR_sub(9,EF_WDATA)  ;  //  step b
	  G_TDO[1] = SDR_sub(33,value) ;
	
		SIR_sub(9,EF_TIMING)  ; //  step c
		SDR_sub(26,0x840022) ;   //  set timing for efuse operation 	
	
		G_TDO[4] = SIR_sub(9,EF_CMD)  ; // step d
	  G_TDO[5] = SDR_sub(4,0x3) ;
	
		HBO_ready = 0 ;    // step e  
	  while(HBO_ready == 0) 
		{
			G_TDO[5] = SIR_sub(9,EF_STATUS)  ; //Step 
			G_TDO[6] = SDR_sub(3,0x1) ;
			if(G_TDO[6] != 0)  HBO_ready = 1 ;
			cnt ++ ;  			
		}	
		// step f - Bring efuse_enable pin low is done by jumper on REB1 
		
		G_TDO[7] = SIR_sub(9,EF_CMD)  ;   //Step g
	  G_TDO[8] = SDR_sub(4,0x7) ;   //   start the write operation
		G_TDO[8] = SDR_sub(4,0x3) ;  //   clear the start operation bit
		VOR_TIM3->CNT_VALUE = 0xFFFFFFFF ;//set TIM3 to reset value(used to measure prog_time)
				
		ctrl_busy = 1 ; 
	  while(ctrl_busy == 1)  //Step h
		{
		iter_til_prog_complete ++  ;  
		if(iter_til_prog_complete > 100) {comm_timeout(3);}//abort if status does not change
		G_TDO[20] = SIR_sub(9,EF_STATUS)  ; 
	  G_TDO[21] = SDR_sub(3,0x1) ;
		if(!(G_TDO[21] & 0x2)) ctrl_busy = 0 ; 
		time_to_prog = 		0xFFFFFFFF - VOR_TIM3->CNT_VALUE ;
		} 
				// step i - Bring efuse_enable pin high is done by jumper on REB1 
		
		G_TDO[11] = SIR_sub(9,EF_CMD)  ; //step j
	  G_TDO[12] = SDR_sub(4,0x0) ;	
		
  	G_TDO[13] =   Read_EF_ADR(1,address); // step k  
		if (EF_ADD_value[0] != value) {comm_timeout(5);}//abort if read does not match	 
    return cnt  ;  
}

 


int main()
{

volatile	uint32_t TDO_string[12],i,k,stat,k2 ;  
	
	/* Enable clock for all peripherals */ 
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE = ( CLK_ENABLE_PORTA | CLK_ENABLE_PORTB | CLK_ENABLE_SPIA | 
																					 CLK_ENABLE_SPIB | CLK_ENABLE_SPIC |
																					 CLK_ENABLE_UARTA | CLK_ENABLE_UARTB | CLK_ENABLE_I2CA |
																					 CLK_ENABLE_I2CB | CLK_ENABLE_IRQSEL |
																					 CLK_ENABLE_IOMGR | CLK_ENABLE_UTILITY | CLK_ENABLE_PORTIO | 
																					 CLK_ENABLE_SYSTEM ); 
	
    CONFIG_TIM3()  ;  //  enable TIM3 - used to measure program time
		
	 	VOR_TIM3->CNT_VALUE = 0xFFFFFFFF ;  //  get TIM3 to reset value 
	
		Reset_JTAG() ;  //  assert TRSTn and move TAP statemachine to parking position
	
	  i = 		0xFFFFFFFF - VOR_TIM3->CNT_VALUE  ;  //  get TIM3 to reset value 
	
	  TDO_string[9] = SDR_sub(32,0x00) ;  //  read out TAP ID # which is available after a RESET

    if(TDO_string[9] != 0x040047e3)  {comm_timeout(9) ; }  // if not read properly, abort 
	
		TDO_string[8] = SIR_sub(9,EF_TIMING)  ; 
		TDO_string[9] = SDR_sub(26,0x840022) ;   //  set timing for efuse operations  
		
    stat =   Read_EF_ADR(16,0); //  read 16 words starting at address 0, results are stored in EF_ADD_value
		
//		k2 = Prog_EF_ADR(0,0x1) ;   //  set index for EF_ID to 1  (pointing to adr 2)
		
//		k2 = Prog_EF_ADR(2,0xF1234) ;  //  prog adr 2 to 0xF1234 
			
//		k2 = Prog_EF_ADR(1,0x3) ;   //  set index for EF_CONFIG to 3  (pointing to adr 3)
		
//		k2 = Prog_EF_ADR(3,0x81400701) ; // prog adr 3 to config value = 0x81400701
			
	  stat  =   Read_EF_ADR(16,0);  //  read 16 words starting at address 0, results are stored in EF_ADD_value
		
 		while(1)  ;    // hang here.  All preceeding code has successfully executed.  
	
}   ///  this is foot of main()
